//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Tuning system plugin
//	Alter note tuning according to selected tuning system
//  To add a tuning system, you need to an element in the combo of tuningSystem.ui
//  Then, you need to define the difference in cents between the degrees of 
//  the "natural midi tuning" and the given tuning system in the tunings array below.
//
//  Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//

//This array contains the difference between natural tuning and tuning systems
var tuning = [];
          //  C  Db   D   Eb  E   F   F#   G   Ab  A   Bb  B 
tuning = [  0, -10,  4, -6, -16, -2, -12, 2, -8,  6,  4, -14 ]; // natural 


//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------

function init()
      {
      // print("test script init");
      }

//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
      {
      var loader = new QUiLoader(null);
      var file   = new QFile(pluginPath + "/TuningMIDICents.ui");
      file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
      form = loader.load(file, null);
      form.allButtons.Apply.connect(accept);
      form.allButtons.temperedPushButton.connect(toTempered);
      form.show();
      }

//---------------------------------------------------------
//    accept
//    called when user presses "Accept" button
//---------------------------------------------------------
function toTempered()
{
  form.whiteKeys.cValue.value = 0;
  form.whiteKeys.dValuel.value = 0;
  form.whiteKeys.eValue.value = 0;
  form.whiteKeys.fValue.value = 0;
  form.whiteKeys.gValue.svalue = 0;
  form.whiteKeys.aValue.value = 0;
  form.whiteKeys.bValue.value = 0;
  form.blackKeys.dbValue.value = 0;
  form.blackKeys.ebValue.value = 0;
  form.blackKeys.gbValue.value = 0;
  form.blackKeys.abValuel.value = 0;
  form.blackKeys.bbValue.value = 0;
}

function accept()
    {
      //get selected tuning
      //var value = form.cbTuning.currentIndex;
      //pick the right tuning in tunings array
      // var tuning = tunings[value];
      
      var cursor = new Cursor(curScore);
      curScore.startUndo();
      for (var staff = 0; staff < curScore.staves; ++staff) {
            cursor.staff = staff;
            for (var v = 0; v < 3; v++) {
              cursor.voice = v;
              cursor.rewind();  // set cursor to first chord/rest

              while (!cursor.eos()) {
                    if (cursor.isChord()) {
                          for (var i = 0; i < cursor.chord().notes(); i++) {
                                var note = cursor.chord().note(i);
                                var pitch = note.pitch % 12;
                                var name = note.name;
                                var idx = getIndex(pitch, name);
                                note.tuning = tuning[idx];
                          }
                    }
                    cursor.next();
              }
            }
       }
      curScore.endUndo();
    }

function getIndex(pitch, name){
    var result;
    switch (pitch) {
    	case 0: //C
    	   result = 0;
    	break;
    	case 1: //C# or Db
    	 result = 1;   
    	break;
    	case 2: //D
    	   result =2;
    	break;
    	case 3: //D# or Eb
    	   result = 3;   
    	break;
    	case 4: //E
    	   result = 4;
    	break;
    	case 5: //F
    	   result = 5;
    	break;
    	case 6: //F# or Gb
    	   result = 6;  
    	break;
    	case 7: //G
    	   result = 7;
    	break;
    	case 8: //G# or Ab
    	   result = 8;  
    	break;
    	case 9: //A
    	   result = 9;
    	break;
    	case 10: //A# or Bb
    	   result = 10;  
    	break;
    	case 11: //B
    	   result = 11;
    	break;
    	default: 
        result =0;
    	break;
  }
  return result;
}


//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.MIDI Cent Tuning',
      init: init,
      run:  run
      };

mscorePlugin;

